import os
import pygame
import sys
import math
import random
import ConfigParser

from pygame.locals import *

sys.path.insert(1,"objects")
sys.path.insert(1,"gui")

sys.path.insert(1, "lib")
from Loader import Loader
from SoundManager import SoundManager
from State import State
from FPSScreen import FPSScreen

sys.path.insert(1, "screens")
from MapScreen import MapScreen
from EscapeScreen import EscapeScreen
from DeathScreen import DeathScreen
from TitleScreen import TitleScreen
from PresentsScreen import PresentsScreen
from SubmitScreen import SubmitScreen
from ScoresScreen import ScoresScreen
from OptionsScreen import OptionsScreen
from HelpScreen import HelpScreen

class game:
	def __init__(self,surface,clock):
		self.loader = Loader()
		self.loader.init_map()
		
		# Default state stuff
		self.state = State()
		self.state.camera = [0,0]
		self.state.font_name = "VeraMono"
		self.init()
		
		self.sound = SoundManager()
		self.sound.init_sound_manager()
		self.state.sound = self.sound
		self.clock = clock
		self.alive = True
		self.screen = surface
		self.width = surface.get_size()[0]
		self.height = surface.get_size()[1]
		
		self.screens = 	[PresentsScreen(100),TitleScreen(),MapScreen(),EscapeScreen(),DeathScreen(),SubmitScreen(),ScoresScreen(),OptionsScreen(),HelpScreen()]
		self.current_screen = 0
		self.screens[self.current_screen].on_focus()
		self.screen_history = []
		self.state.switch_to_screen = self.move_to
		self.state.previous_screen = self.previous_screen
		self.state.reset = self.init
		
		self.state.cursor_coords = (0,0)
		
		# TODO: move this into configparser file
		#  plus others!
		self.state.pause_on_action = True
		
		self.fps = FPSScreen(clock)
		pygame.mouse.set_visible(False)
		
	def init(self):
		self.state.levels = []
		self.state.level = 0
		self.state.gold = 0
		self.state.fuel = 100
		self.state.max_level = 0
		
	def reset(self):
		self.move_to(TitleScreen)
		self.state.camera = [0,0]
	
	def previous_screen(self):
		if len(self.screen_history) <= 0:
			self.current_screen -= 1
		else:
			self.current_screen = self.screen_history.pop()
		self.screens[self.current_screen].on_focus()
	
	def move_to(self,klass):
		i = 0
		for scrn in self.screens:
			if isinstance(scrn, klass):
				self.screen_history.append(self.current_screen)
				self.current_screen = i
			i += 1
		self.screens[self.current_screen].on_focus()
	
	def draw(self):
		# INCREDIBLY wasteful...
		self.screen.fill( (255,255,255) )
		self.screens[self.current_screen].draw(self.screen)
#		self.fps.draw(self.screen)
#		self.screen.blit(self.state.cursor,(self.state.cursor_coords[0]-8,self.state.cursor_coords[1]-8))
		return True
		
	def handle_event(self,event):
		if event.type == QUIT:
			self.alive = False
		elif event.type == MOUSEMOTION:
			self.state.cursor_coords = event.pos
		try:
			self.alive = self.screens[self.current_screen].handle_event(event)
		except AttributeError:
			True
	
	def update(self):
		# If the current screen returns FALSE we should move to the next screen in the list.
		retval = self.screens[self.current_screen].update()
		self.fps.update()
		if retval == False:
			if self.current_screen < len(self.screens)-1:
				self.current_screen += 1
				self.screens[self.current_screen].on_focus()
				return True
			else:
				return False
		else:
			return True

