import pygame
import pygame.gfxdraw
from pygame.locals import *
from StaticText import StaticText
from Loader import Loader
from Widget import Widget
from State import State
from Loader import Loader
import util
import math

class ActionWheel(Widget):
	# actions should be a list of tuples, each correspoding to a verb:
	#   [("Do Something","do_something.png",object.do_something)...]
	def __init__(self,x,y,actions):
		w = 300
		h = 300
		Widget.__init__(self,x,y,w,h)
		self.slices = []
		for i in range(0,8):
			self.slices.append(None)
		# Load slices & actions
		self.actions = actions
		i = 0
		for j in range(0,len(actions)):
			self.slices[i] = ActionSlice(i,actions[j])
			i += int(9/len(actions)+0.5)
			
		self.surface = pygame.surface.Surface((w,h))
		self.surface.set_colorkey( (0,0,0) )
		self.surface.set_alpha(16)
		for slice in self.slices:
			if slice:
				slice.draw(self.surface)
				slice.set_alpha(64)
		self.active = -1
		self.state = State()
		self.label = None
		self.sticky = False
	
	def selected(self):
		if self.slices[self.active]:
			return self.slices[self.active].action[2]
		return None
	
	def update(self):
		angle = util.radians_to_angle(math.atan2( (self.y-self.state.cursor_coords[1]),(self.state.cursor_coords[0]-self.x) ))
		angle += 115
		if angle < 0:
			angle = 360+angle
		active = int(angle/45)
		if active != self.active:
			self.active = active
			if  self.slices[active]:
				self.label = StaticText(self.x-self.size[0]/2,self.y-16,self.slices[active].action[0],(196,196,196),32)
				self.label.align("center",self.size[0])
				self.label.shadow(True)
	
	def click(self,x,y):
		return self.selected()
	
	def draw(self,surf):
		pos = (self.x-self.size[0]/2,self.y-self.size[1]/2)
		surf.blit(self.surface,pos)
		if self.slices[self.active]:
			self.slices[self.active].draw(surf,pos)
			if self.label:
				self.label.draw(surf)
		for slice in self.slices:
			if slice:
				angle = util.angle_to_radians(slice.angle/8.0*360)
				x = self.x + math.sin(angle)*100
				y = self.y + math.cos(angle)*100
				slice.action[1].set_colorkey( (255,255,255) )
				surf.blit(slice.action[1],(x-16,y-16))

basic_slice = [None,None]

class ActionSlice:
	def __init__(self,angle,action,icon=None):
		global basic_slice
		if basic_slice[0] == None:
			basic_slice[0] = self.generate_basic_slice(0)
			basic_slice[1] = self.generate_basic_slice(1)
		self.surface = pygame.transform.rotate(basic_slice[angle%2],angle/2*90)
		self.angle = angle
		self.action = (action[0],Loader().load_art(action[1])[0],action[2])
		self.action[1].set_colorkey( (0,0,0) )
	
	def set_alpha(self,alpha):
		self.surface.set_alpha(alpha)
	
	def draw(self,surf,pos=(0,0)):
		surf.blit(self.surface,pos)
	
	def generate_basic_slice(self,seg):
		size = 300
		surface = pygame.surface.Surface((size*2,size*2))
		pygame.gfxdraw.filled_circle(surface,size,size,size,(255,255,255))
		pygame.gfxdraw.aacircle(surface,size,size,size,(255,255,255))
		points = None
		if seg == 0:
			points = [(0,0),(0,size*2),(size*0.65,size*2),(size,size),(size*1.35,size*2),(size*2,size*2),(size*2,0)]
		elif seg == 1:
			points = [(0,0),(0,size*2),(size*1.5,size*2),(size,size),(size*2,size*1.5),(size*2,0)]
		pygame.gfxdraw.filled_polygon(surface,points,(0,0,0))
		surface = pygame.transform.smoothscale(surface,(size,size))
		surface.set_colorkey( (0,0,0) )
		return surface