import pygame
from State import State
import util
from Loader import Loader
from Widget import Widget

class Minimap(Widget):
	def __init__(self):
		self.state = State()
		Widget.__init__(self,16,self.state.screen_size[1]-144,128,128)
		self.player = self.state.player
		self.surface = pygame.surface.Surface( self.size )
		self.scale = 1000.0
		self.font = Loader().load_font("slkscr",8)
	
	def redraw(self):
		self.surface.fill( (0,0,0) )
		# Draw radar blips.
		for obj in self.state.space:
			if hasattr(obj,'children'):
				for child in obj.children():
					self.draw_object(child)
			else:
				self.draw_object(obj)
		pygame.draw.rect(self.surface,(64,64,64),pygame.Rect(0,0,self.size[0],self.size[1]),1)
		self.surface.set_alpha(196)
	
	def draw_object(self,entity):	
		distance = util.distance(entity,self.player)/2
		if distance < self.scale and hasattr(entity,"minimap_color"):
			x = (entity.x-self.player.x)/self.scale * self.size[0]/2 + self.size[0]/2
			y = (entity.y-self.player.y)/self.scale * self.size[1]/2 + self.size[1]/2
			pygame.draw.circle(self.surface,entity.minimap_color,(x,y),entity.minimap_size)
			if entity.minimap_label == None and hasattr(entity,"name"):
				entity.minimap_label = self.font.render(entity.name,False,entity.minimap_color)
			elif entity.minimap_label != None:
				self.surface.blit(entity.minimap_label,(x+2,y-8))
	
	def update(self):
		self.redraw()
		
	def draw(self,surface):
		surface.blit(self.surface,self.coords)