from GuiManager import GuiManager
from Widget import Widget
from TextButton import TextButton
from State import State
from StaticText import StaticText
from Line import Line
import pygame

class Overlay(Widget):
	textcolor = (196,196,196)

	def __init__(self,title,padding=32):
		self.state = State()
		Widget.__init__(self,padding,padding,self.state.screen_size[0]-2*padding,self.state.screen_size[1]-2*padding)
		self.children = GuiManager()
		self.padding = padding
		self.alive = True
		self.close_button = TextButton(0,0,"X",self.close)
		self.close_button.x = self.size[0]-self.close_button.size[0]-2
		self.close_button.passive_color = (128,128,128)
		self.add(self.close_button)
		self.title = title
		self.state = State()
		self.render()
	
	def render(self):
		self.clear()
		title_s = StaticText(0,5,self.title,self.textcolor,16)
		title_s.align("center",self.size[0])
		self.add(title_s)
		self.add(Line((7,title_s.size[1]+4),(self.size[0]-10,title_s.size[1]+4),self.textcolor,2))
		self.start_y = title_s.y + title_s.size[1]+5
	
	def close(self):
		self.alive = False
	
	def add(self,widget):
		self.children.add(widget)
	
	def remove(self,widget):
		self.children.remove(widget)
	
	def clear(self):
		self.children.clear()
		self.add(self.close_button)
	
	def update(self):
		self.children.update()
		return self.alive
	
	def draw(self,surf):
		self.surface = pygame.surface.Surface( self.size )
		self.surface.fill( (0,0,0) )
		self.surface.set_alpha( 196 )
		self.children.draw(self.surface)
		pygame.draw.rect(self.surface,(128,128,128),pygame.Rect(0,0,self.size[0]-1,self.size[1]-1),2)
		surf.blit(self.surface,(self.x,self.y))
	
	def mouseover(self,x,y):
		if self.rect.collidepoint(x,y):
			x -= self.padding
			y -= self.padding
			self.children.mouseover(x,y)
			return True
	
	def click(self,x,y):
		if self.rect.collidepoint(x,y):
			x -= self.padding
			y -= self.padding
			self.children.click(x,y)
			return True