import pygame
from State import State
from GuiManager import GuiManager

class AbstractScreen:
	# Create a new AbstractScreen. Don't do this, obviously.
	def __init__(self):
		self.previous = None
		self.widgets = GuiManager()
		self.state = State()
	
	# Call this when the screen is switched to. Can we make this automatic?
	def on_focus(self):
		raise Exception("Abstract on_focus() method called")
	
	# Draw this screen, returning True on success
	def draw(self,screen):
		self.widgets.draw(screen)
		return True

	def add(self,widget):
		self.widgets.add(widget)
	
	def remove(self,widget):
		self.widgets.remove(widget)
	
	def clear(self):
		self.widgets.clear()
	
	# Return value:
	#	True -- successful update
	#	False -- Error in update OR done, and switch to previous screen
	def update(self):
		self.widgets.update()
		return True