import pygame
from State import State
from Loader import Loader
from Widget import Widget

class HexMap(Widget):
	# map should be a two-dimensional array
	def __init__(self,x,y,map):
		self.map = map
		width = len(self.map[0])
		height = len(self.map)
		self.tiles = Loader().load_art("tiles.png")[0]
		size = self.tiles.get_size()
		self.tile_count = (4,4)
		self.tile_rect = pygame.Rect(0,0,size[0]/self.tile_count[0],size[1]/self.tile_count[1])
		self.state = State()
		Widget.__init__(self,x,y,width*(self.tile_rect.w+self.tile_rect.w/2) - self.tile_rect.width/3,(height-1.5)*(self.tile_rect.h-self.tile_rect.h/2))
		self.surface = pygame.surface.Surface(self.size)
		self.surface.fill( (255,255,255) )
		self.fog = pygame.surface.Surface(self.size)
		self.fog.fill( (255,255,255) )
		self.darkness = pygame.surface.Surface(self.size)
		self.darkness.fill( (1,1,1) )
		self.render(False)
		self.render_fog()
	
	def point_to_coords(self,x,y):
		_x = x*(self.tile_rect.width + self.tile_rect.width/2-2)
		if y % 2 == 1:
			_x += self.tile_rect.width/4*3 - 1
		_x += self.tile_rect.w/2
		_y = y*(self.tile_rect.height/2-0) + self.tile_rect.height/2
		return (_x,_y)
	
	def move_tile_rect(self,index):
		x = index % self.tile_count[0] * self.tile_rect.w
		y = index / self.tile_count[1] * self.tile_rect.h
		self.tile_rect.x = x
		self.tile_rect.y = y
	
	def tile(self,x,y):
		return self.map[y][x][0]
	
	def draw(self,surf):
		coords = self.state.offset(self.x,self.y)
		surf.blit(self.surface,coords)
	
	def draw_fog(self,surf):
		coords = self.state.offset(self.x,self.y)
		surf.blit(self.fog,coords,None,pygame.BLEND_SUB)
	
	def render_fog(self,center=None):
		# grey out old fog
		self.fog.blit(self.darkness,(0,0),None,pygame.BLEND_ADD)
		_x = 0
		y = 0
		row_count = 0
		col_count = 0
		rows = range(0,len(self.map))
		cols = range(0,len(self.map[0]))
		if center:
			#cols = range(0,12)
			cols = range(center[0]-2,center[0]+2)
			rows = range(center[1]-4,center[1]+5)
		for row in self.map:
			x = _x
			if row_count % 2 == 1:
				x += self.tile_rect.width/4*3 - 1
			col_count = 0
			for index in row:
				if index[1] != -1 and row_count in rows and col_count in cols:
					self.move_tile_rect(index[1]+12)
					self.fog.blit(self.tiles,(x,y),self.tile_rect)
					if index[0] == 0:
						self.darkness.blit(self.tiles,(x,y),self.tile_rect)
				x += self.tile_rect.width + self.tile_rect.width/2 - 2
				col_count += 1
			y += self.tile_rect.height/2
			if y >= self.size[1]:
				y = 0
			row_count += 1
	
	def render(self,debug = False):
		_x = 0
		y = 0
		row_count = 0
		col_count = 0
		for row in self.map:
			x = _x
			if row_count % 2 == 1:
				x += self.tile_rect.width/4*3 - 1
			col_count = 0
			for index in row:
				self.move_tile_rect(index[0])
				self.surface.blit(self.tiles,(x,y),self.tile_rect)
				# label tiles for debugging
				if debug:
					label = Loader().load_font("slkscr",16).render(str(col_count)+","+str(row_count),False,(0,0,0))
					label.set_alpha(32)
					self.surface.blit(label,(x+self.tile_rect.w/2-label.get_size()[0]/2,y+self.tile_rect.h/2-label.get_size()[1]/2))
				x += self.tile_rect.width + self.tile_rect.width/2 - 2
				col_count += 1
			y += self.tile_rect.height/2
			if y >= self.size[1]:
				y = 0
			row_count += 1