import pygame
from pygame.locals import *
from AbstractScreen import AbstractScreen
import TitleScreen
from EscapeScreen import EscapeScreen
from DeathScreen import DeathScreen

import random
import MapGenerator
from HexMap import HexMap
from Entity import Entity
from Player import Player
from StaticText import StaticText
from GuiManager import GuiManager

class MapScreen(AbstractScreen):
	def __init__(self):
		AbstractScreen.__init__(self)
	
	def on_focus(self):
		if self.state.level == 0:
			self.state.sound.start_music(self.state.sound.get_track_number("Paranoia"))
		if self.state.level > self.state.max_level:
			self.state.max_level = self.state.level
		self.clear()
		self.overlay = GuiManager()
		self.status = StaticText(5,self.state.screen_size[1]-21," ",(255,255,255),16)
		self.zorkmids = StaticText(5,self.status.y-self.status.size[1]-5,"$"+str(self.state.gold),(64,192,64),16)
		self.fuel = StaticText(5,self.zorkmids.y-self.status.size[1]-5,"Fuel: "+str(self.state.fuel),(255,255,64),16)
		self.level = StaticText(5,self.fuel.y-self.status.size[1]-5,"Level: "+str(self.state.level+1),(192,192,192),16)
		self.overlay.add(self.status)
		self.overlay.add(self.zorkmids)
		self.overlay.add(self.fuel)
		self.overlay.add(self.level)
		if self.state.level >= len(self.state.levels):
			level = MapGenerator.generate(20,80)
			self.state.levels.append(level)
			spawn = level[1]['up']
			self.status.set_text("You descend the stairs into the unknown...")
		else:
			level = self.state.levels[self.state.level]
			spawn = self.state.spawn
		self.map = HexMap(0,0,level[0])
		self.add(self.map)
		self.state.camera = [self.map.size[0]/2,self.map.size[1]/2]
		
		# Event handling crud
		self.keys = {}
		self.keys[K_LEFT] = False
		self.keys[K_RIGHT] = False
		self.keys[K_UP] = False
		self.keys[K_DOWN] = False
		self.keys[1] = False
		self.mouse_coords = [0,0]
		self.grab_coords = [0,0]
		
		# Drop gold
		self.gold = []
		for pos in level[1]['money']:
			gold = Entity("$",(64,192,0),self.map,pos)
			self.gold.append(gold)
			self.add(gold)
		# Drop wood
		self.wood= []
		for pos in level[1]['fuel']:
			wood = Entity("#",(148,105,45),self.map,pos)
			self.wood.append(wood)
			self.add(wood)
		# Add stairs
		self.down = Entity(">",(32,0,0),self.map,level[1]['down'])
		self.add(self.down)
		self.up = Entity("<",(32,0,0),self.map,level[1]['up'])
		self.add(self.up)
		# Spawn player
		self.player = Player(self.map,spawn)
		self.add(self.player)
	
	def go_up(self):
		self.state.level -= 1
		if self.state.level < 0:
			self.state.switch_to_screen(EscapeScreen)
		else:
			self.state.spawn = self.state.levels[self.state.level][1]['down']
			self.state.switch_to_screen(MapScreen)
	
	def go_down(self):
		self.state.level += 1
		self.state.switch_to_screen(MapScreen)
	
	def draw(self,screen):
		AbstractScreen.draw(self,screen)
		self.map.draw_fog(screen)
		self.overlay.draw(screen)
		
	def do_pickup(self):
		# cheap handling of picking up objects...
		to_remove = None
		for gold in self.gold:
			if gold.coords[0] == self.player.coords[0] and gold.coords[1] == self.player.coords[1]:
				amt = random.randint(10,100)
				self.state.gold += amt
				self.status.set_text("Picked up "+str(amt)+" zorkmids")
				self.zorkmids.set_text("$"+str(self.state.gold))
				to_remove = gold
				break
		if to_remove:
			self.gold.remove(to_remove)
			self.state.levels[self.state.level][1]['money'].remove((to_remove.coords[0],to_remove.coords[1]))
			self.remove(to_remove)
		if to_remove:
			to_remove = False
		for wood in self.wood:
			if wood.coords[0] == self.player.coords[0] and wood.coords[1] == self.player.coords[1]:
				amt = random.randint(30,50)
				self.state.fuel += amt
				self.status.set_text("Picked up some wood")
				to_remove = wood
				break
		if to_remove:
			self.wood.remove(to_remove)
			self.state.levels[self.state.level][1]['fuel'].remove((to_remove.coords[0],to_remove.coords[1]))
			self.remove(to_remove)
		if to_remove == None:
			self.status.set_text(" ")
		self.fuel.set_text("Fuel: "+str(int(self.state.fuel)))
		if self.state.fuel <= 0:
			self.state.switch_to_screen(DeathScreen)
	
	def handle_event(self,event):
		if event.type == KEYDOWN:
			if event.key == K_ESCAPE:
				self.state.switch_to_screen(TitleScreen.TitleScreen)
			elif event.key == K_LEFT:
				self.keys[K_LEFT] = True
			elif event.key == K_RIGHT:
				self.keys[K_RIGHT] = True
			elif event.key == K_UP:
				self.keys[K_UP] = True
			elif event.key == K_DOWN:
				self.keys[K_DOWN] = True
			elif event.key == 119: # w
				self.player.move(0)
			elif event.key == 101: # e
				self.player.move(1)
			elif event.key == 100: # d
				self.player.move(2)
			elif event.key == 115: # s
				self.player.move(3)
			elif event.key == 97: # a
				self.player.move(4)
			elif event.key == 113: # q
				self.player.move(5)
			elif event.unicode == '<':
				if self.player.coords[0] == self.up.coords[0] and self.player.coords[1] == self.up.coords[1]:
					self.go_up()
			elif event.unicode == '>':
				if self.player.coords[0] == self.down.coords[0] and self.player.coords[1] == self.down.coords[1]:
					self.go_down()
			self.do_pickup()
		elif event.type == KEYUP:
			if event.key == K_LEFT:
				self.keys[K_LEFT] = False
			elif event.key == K_RIGHT:
				self.keys[K_RIGHT] = False
			elif event.key == K_UP:
				self.keys[K_UP] = False
			elif event.key == K_DOWN:
				self.keys[K_DOWN] = False
		elif event.type == MOUSEBUTTONDOWN:
			self.keys[event.button] = True
			self.grab_coords = event.pos
		elif event.type == MOUSEBUTTONUP:
			self.keys[event.button] = False
		elif event.type == MOUSEMOTION:
			self.mouse_coords = event.pos
		return True
	
	# Return value:
	#	True -- successful update
	#	False -- Error in update OR done, and switch to previous screen
	def update(self):
		if self.keys[K_LEFT]:
			self.state.camera[0] -= 10
		elif self.keys[K_RIGHT]:
			self.state.camera[0] += 10
		if self.keys[K_UP]:
			self.state.camera[1] -= 10
		elif self.keys[K_DOWN]:
			self.state.camera[1] += 10
		if self.keys[1]: # left mouse button down
			self.state.camera[0] += (self.grab_coords[0]-self.mouse_coords[0])
			self.state.camera[1] += (self.grab_coords[1]-self.mouse_coords[1])
			self.grab_coords = self.mouse_coords
		map_offset = self.state.offset(self.map.x,self.map.y)
		#self.screen_size[0]/2 + x - self.camera[0]
		if map_offset[0] > 0:
			camera_x = self.state.screen_size[0]/2 + self.map.x
			self.state.camera[0] = camera_x
		elif map_offset[0] + self.map.size[0] < self.state.screen_size[0]:
			camera_x = self.state.screen_size[0]/2 + self.map.x + (self.map.size[0] - self.state.screen_size[0])
			self.state.camera[0] = camera_x
		if map_offset[1] > 0:
			camera_y = self.state.screen_size[1]/2 + self.map.y
			self.state.camera[1] = camera_y
		elif map_offset[1] + self.map.size[1] < self.state.screen_size[1]:
			camera_x = self.state.screen_size[1]/2 + self.map.y + (self.map.size[1] - self.state.screen_size[1])
			self.state.camera[1] = camera_x
		self.overlay.update()
		return AbstractScreen.update(self)