from AbstractScreen import AbstractScreen
import sys
from pygame.locals import *
import os
import pygame
from Loader import Loader
from State import State
from StaticText import StaticText
from HexMap import HexMap
import MapGenerator
import random
import MapScreen
import ScoresScreen
import OptionsScreen
import HelpScreen
from Player import Player

class TitleScreen(AbstractScreen):
	def __init__(self):
		AbstractScreen.__init__(self)
		self.loader = Loader()
		self.state = State()
		x = self.state.screen_size[0]/2
		y = self.state.screen_size[1]/6
		stitle = ["...Deeper"]
		self.title = []
		for title in stitle:
			lbl = StaticText(0,y,title,(255,255,255),64)
			self.title.append( lbl )
			lbl.align("center")
			lbl.shadow()
		smenu = ["Start","Help","Options","High Scores","Quit"]
		self.menuitems = []
		y += self.title[0].size[1]
		x = 0
		self.selected = 0
		for menu in smenu:
			lbl = StaticText(x,y,menu,(128,128,128),32)
			lbl2 = StaticText(x,y,menu,(255,255,255),32)
			x += lbl.size[0]+40
			lbl.shadow()
			lbl2.shadow()
			self.menuitems.append((lbl,lbl2))
		startx = self.state.screen_size[0]/2-x/2
		for menu in self.menuitems:
			menu[0].x = (menu[0].coords[0] + startx)
			menu[1].x = (menu[1].coords[0] + startx)
		
		map = HexMap(0,0,MapGenerator.generate(60,40,(30,20))[0])
		map.x = -map.width/2
		map.y = -map.height/2+self.state.screen_size[1]/3
		player = Player(map,(30,10))
		self.player = player
		self.map = map
		self.add(map)
		self.shadow = pygame.surface.Surface(self.state.screen_size)
		self.shadow.fill( (0,0,0) )
		self.shadow.set_alpha(192)
		
		# Set up fade timer
		self.ttl = 255
		self.tick = 0
	
	def on_focus(self):
		self.alive = True
		track = self.state.sound.get_track_number("Rock Rat")
		if self.state.sound.last_played_music != track:
			self.state.sound.fade_music()
			self.state.sound.start_music(track)
		
	def update(self):
		AbstractScreen.update(self)
		if pygame.time.get_ticks() > self.tick:
			self.player.move(random.randint(0,5))
			self.tick = pygame.time.get_ticks()+100
		return True
	
	def handle_event(self,event):
		if event.type == pygame.locals.KEYDOWN:
			if event.key == K_LEFT or event.key == K_UP:
				self.selected -= 1
				if self.selected < 0:
					self.selected = len(self.menuitems)-1
			elif event.key == K_RIGHT or event.key == K_DOWN:
				self.selected += 1
				if self.selected > len(self.menuitems)-1:
					self.selected = 0
			elif event.key == K_RETURN:
				if self.selected == 0:
					self.state.reset()
					self.state.sound.fade_music()
					self.state.switch_to_screen(MapScreen.MapScreen)
				elif self.selected == 1:
					self.state.switch_to_screen(HelpScreen.HelpScreen)
				elif self.selected == 2:
					self.state.switch_to_screen(OptionsScreen.OptionsScreen)
				elif self.selected == 3:
					self.state.switch_to_screen(ScoresScreen.ScoresScreen)
				elif self.selected == 4:
					sys.exit()
		elif event.type == pygame.locals.MOUSEBUTTONDOWN:
			self
		return True
	
	def draw(self,screen):
		screen.fill( (0,0,0) )
		AbstractScreen.draw(self,screen)
		self.map.draw_fog(screen)
		screen.blit(self.shadow,(0,0))
		sel = self.menuitems[self.selected][0]
	#	pygame.draw.rect(screen,(196,196,196),pygame.Rect(sel.coords[0]-2,sel.coords[1]-1,sel.width+4,sel.height+2))
		i = 0
		for label in self.menuitems:
			if i == self.selected:
				label[1].draw(screen)
			else:
				label[0].draw(screen)
			i += 1
		for label in self.title:
			label.draw(screen)
		return True
